@@ -147,7 +147,8 @@ namespace sf2 {
         ::LoadString(ck, s); // function from helper.h
     }
 
-    Sample::Sample(RIFF::Chunk* ck, RIFF::Chunk* pCkSmpl, RIFF::Chunk* pCkSm24) {
+    Sample::Sample(File* file, RIFF::Chunk* ck, RIFF::Chunk* pCkSmpl, RIFF::Chunk* pCkSm24) {
+        this->pFile = file;
         this->pCkSmpl = pCkSmpl;
         this->pCkSm24 = pCkSm24;
 
@@ -728,13 +729,13 @@ namespace sf2 {
 
     InstrumentBase::~InstrumentBase() {
         if (pGlobalRegion) delete pGlobalRegion;
-        for (int i = regions.size() - 1; i >= 0; i--) {
+        for (ssize_t i = regions.size() - 1; i >= 0; i--) {
             if (regions[i]) delete (regions[i]);
         }
     }
 
     int InstrumentBase::GetRegionCount() {
-        return regions.size();
+        return (int) regions.size();
     }
 
     Region* InstrumentBase::GetRegion(int idx) {
@@ -1012,7 +1013,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken phdr)");
         }
 
-        int count = ck->GetSize() / 38;
+        int count = (int) ck->GetSize() / 38;
         for (int i = 0; i < count; i++) {
             Presets.push_back(new Preset(this, ck));
         }
@@ -1022,7 +1023,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken pbag)");
         }
 
-        count = ck->GetSize() / 4;
+        count = int(ck->GetSize() / 4);
         for (int i = 0; i < count; i++) {
             PresetBag pb;
             pb.GenNdx = ck->ReadInt16();
@@ -1036,7 +1037,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken pmod)");
         }
 
-        count = ck->GetSize() / 10;
+        count = int(ck->GetSize() / 10);
         for (int i = 0; i < count; i++) {
             ModList ml;
             ml.ModSrcOper = ck->ReadInt16();
@@ -1053,7 +1054,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken pgen)");
         }
 
-        count = ck->GetSize() / 4;
+        count = int(ck->GetSize() / 4);
         for (int i = 0; i < count; i++) {
             GenList gl;
             gl.GenOper = ck->ReadInt16();
@@ -1066,7 +1067,7 @@ namespace sf2 {
         if (ck->GetSize() < (22 * 2) || (ck->GetSize() % 22)) {
             throw Exception("Broken SF2 file (broken inst)");
         }
-        count = ck->GetSize() / 22;
+        count = int(ck->GetSize() / 22);
         for (int i = 0; i < count; i++) {
             Instruments.push_back(new Instrument(this, ck));
         }
@@ -1076,7 +1077,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken ibag)");
         }
 
-        count = ck->GetSize() / 4;
+        count = int(ck->GetSize() / 4);
         for (int i = 0; i < count; i++) {
             InstBag ib;
             ib.InstGenNdx = ck->ReadInt16();
@@ -1090,7 +1091,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken imod)");
         }
 
-        count = ck->GetSize() / 10;
+        count = int(ck->GetSize() / 10);
         for (int i = 0; i < count; i++) {
             ModList ml;
             ml.ModSrcOper = ck->ReadInt16();
@@ -1107,7 +1108,7 @@ namespace sf2 {
             throw Exception("Broken SF2 file (broken igen)");
         }
 
-        count = ck->GetSize() / 4;
+        count = int(ck->GetSize() / 4);
         for (int i = 0; i < count; i++) {
             GenList gl;
             gl.GenOper = ck->ReadInt16();
@@ -1120,9 +1121,9 @@ namespace sf2 {
         if ((ck->GetSize() % 46)) {
             throw Exception("Broken SF2 file (broken shdr)");
         }
-        count = ck->GetSize() / 46;
+        count = int(ck->GetSize() / 46);
         for (int i = 0; i < count; i++) {
-            Samples.push_back(new Sample(ck, pCkSmpl, pCkSm24));
+            Samples.push_back(new Sample(this, ck, pCkSmpl, pCkSm24));
         }
 
         // Loading instrument regions
@@ -1152,19 +1153,19 @@ namespace sf2 {
 
     File::~File() {
         delete pInfo;
-        for (int i = Presets.size() - 1; i >= 0; i--) {
+        for (ssize_t i = Presets.size() - 1; i >= 0; i--) {
             if (Presets[i]) delete (Presets[i]);
         }
-        for (int i = Instruments.size() - 1; i >= 0; i--) {
+        for (ssize_t i = Instruments.size() - 1; i >= 0; i--) {
             if (Instruments[i]) delete (Instruments[i]);
         }
-        for (int i = Samples.size() - 1; i >= 0; i--) {
+        for (ssize_t i = Samples.size() - 1; i >= 0; i--) {
             if (Samples[i]) delete (Samples[i]);
         }
     }
 
     int File::GetPresetCount() {
-        return Presets.size() - 1; // exclude terminal preset (EOP)
+        return (int) Presets.size() - 1; // exclude terminal preset (EOP)
     }
 
     Preset* File::GetPreset(int idx) {
@@ -1176,7 +1177,7 @@ namespace sf2 {
     }
 
     int File::GetInstrumentCount() {
-        return Instruments.size() - 1; // exclude terminal instrument (EOI)
+        return (int) Instruments.size() - 1; // exclude terminal instrument (EOI)
     }
 
     Instrument* File::GetInstrument(int idx) {
@@ -1188,6 +1189,8 @@ namespace sf2 {
     }
 
     void File::DeleteInstrument(Instrument* pInstrument) {
+        if (!pInstrument) return;
+
         for (int i = 0; i < GetPresetCount(); i++) {
             Preset* p = GetPreset(i);
             if (p == NULL) continue;
@@ -1202,12 +1205,16 @@ namespace sf2 {
             if (GetInstrument(i) == pInstrument) {
                 Instruments[i] = NULL;
                 delete pInstrument;
+                // an instrument instance only exists once in the list, so stop
+                // here (which also silences a clang sanatizer warning about
+                // potential multiple memory releases of pInstrument above)
+                return;
             }
         }
     }
 
     int File::GetSampleCount() {
-        return Samples.size() - 1; // exclude terminal sample (EOS)
+        return (int) Samples.size() - 1; // exclude terminal sample (EOS)
     }
 
     Sample* File::GetSample(int idx) {
@@ -1251,6 +1258,10 @@ namespace sf2 {
         return false;
     }
 
+    RIFF::File* File::GetRiffFile() {
+        return pRIFF;
+    }
+
     /**
      * Loads the whole sample wave into RAM. Use
      * ReleaseSampleData() to free the memory if you don't need the cached
@@ -1425,7 +1436,7 @@ namespace sf2 {
             if (pSample->SampleType == Sample::MONO_SAMPLE || pSample->SampleType == Sample::ROM_MONO_SAMPLE) {
                 pSample->pCkSmpl->Read(pTmpBuf, SampleCount, 2);
                 pSample->pCkSm24->Read(pTmpBuf + SampleCount * 2, SampleCount, 1);
-                for (int i = SampleCount - 1; i >= 0; i--) {
+                for (long i = SampleCount - 1; i >= 0; i--) {
                     pBuf[i*3]     = pTmpBuf[(SampleCount * 2) + i];
                     pBuf[i*3 + 2] = pTmpBuf[i*2 + 1];
                     pBuf[i*3 + 1] = pTmpBuf[i*2];
@@ -1433,7 +1444,7 @@ namespace sf2 {
             } else if (pSample->SampleType == Sample::LEFT_SAMPLE || pSample->SampleType == Sample::ROM_LEFT_SAMPLE) {
                 pSample->pCkSmpl->Read(pTmpBuf, SampleCount, 2);
                 pSample->pCkSm24->Read(pTmpBuf + SampleCount * 2, SampleCount, 1);
-                for (int i = SampleCount - 1; i >= 0; i--) {
+                for (long i = SampleCount - 1; i >= 0; i--) {
                     pBuf[i*6]     = pTmpBuf[(SampleCount * 2) + i];
                     pBuf[i*6 + 2] = pTmpBuf[i*2 + 1];
                     pBuf[i*6 + 1] = pTmpBuf[i*2];
@@ -1443,7 +1454,7 @@ namespace sf2 {
             } else if (pSample->SampleType == Sample::RIGHT_SAMPLE || pSample->SampleType == Sample::ROM_RIGHT_SAMPLE) {
                 pSample->pCkSmpl->Read(pTmpBuf, SampleCount, 2);
                 pSample->pCkSm24->Read(pTmpBuf + SampleCount * 2, SampleCount, 1);
-                for (int i = SampleCount - 1; i >= 0; i--) {
+                for (long i = SampleCount - 1; i >= 0; i--) {
                     pBuf[i*6 + 3] = pTmpBuf[(SampleCount * 2) + i];
                     pBuf[i*6 + 5] = pTmpBuf[i*2 + 1];
                     pBuf[i*6 + 4] = pTmpBuf[i*2];
@@ -1460,14 +1471,14 @@ namespace sf2 {
             int16_t* const pBuf    = (int16_t*) pBuffer;
             if (pSample->SampleType == Sample::LEFT_SAMPLE || pSample->SampleType == Sample::ROM_LEFT_SAMPLE) {
                 pSample->pCkSmpl->Read(pTmpBuf, SampleCount, 2);
-                for (int i = SampleCount - 1; i >= 0; i--) {
+                for (long i = SampleCount - 1; i >= 0; i--) {
                     pBuf[i*2] = pTmpBuf[i];
                     if (CLEAR)
                         pBuf[i*2 + 1] = 0;
                 }
             } else if (pSample->SampleType == Sample::RIGHT_SAMPLE || pSample->SampleType == Sample::ROM_RIGHT_SAMPLE) {
                 pSample->pCkSmpl->Read(pTmpBuf, SampleCount, 2);
-                for (int i = SampleCount - 1; i >= 0; i--) {
+                for (long i = SampleCount - 1; i >= 0; i--) {
                     if (CLEAR)
                         pBuf[i*2] = 0;
                     pBuf[i*2 + 1] = pTmpBuf[i];
