@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2014 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2019 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This program is part of libgig.                                       *
@@ -37,46 +37,103 @@ using namespace std;
 
 string Revision();
 void PrintVersion();
-void PrintFileInformations(gig::File* gig);
+void PrintFileInformation(gig::File* gig);
 void PrintGroups(gig::File* gig);
 void PrintSamples(gig::File* gig);
 void PrintScripts(gig::File* gig);
 void PrintInstruments(gig::File* gig);
+void PrintInstrumentNamesOnly(gig::File* gig);
 void PrintRegions(gig::Instrument* instr);
 void PrintUsage();
 void PrintDimensionRegions(gig::Region* rgn);
+bool VerifyFile(gig::File* gig);
+void RebuildChecksumTable(gig::File* gig);
+
+class PubSample : public gig::Sample {
+public:
+    using DLS::Sample::pCkData;
+};
+
+class PubFile : public gig::File {
+public:
+    using gig::File::VerifySampleChecksumTable;
+    using gig::File::RebuildSampleChecksumTable;
+};
+
 
 int main(int argc, char *argv[])
 {
+    bool bVerify = false;
+    bool bRebuildChecksums = false;
+    bool bInstrumentNamesOnly = false;
+
     if (argc <= 1) {
         PrintUsage();
         return EXIT_FAILURE;
     }
-    if (argv[1][0] == '-') {
-        switch (argv[1][1]) {
-            case 'v':
-                PrintVersion();
-                return EXIT_SUCCESS;
+
+    int iArg;
+    for (iArg = 1; iArg < argc; ++iArg) {
+        const string opt = argv[iArg];
+        if (opt == "--") { // common for all command line tools: separator between initial option arguments and i.e. subsequent file arguments
+            iArg++;
+            break;
+        }
+        if (opt.substr(0, 1) != "-") break;
+
+        if (opt == "-v") {
+            PrintVersion();
+            return EXIT_SUCCESS;
+        } else if (opt == "--verify") {
+            bVerify = true;
+        } else if (opt == "--rebuild-checksums") {
+            bRebuildChecksums = true;
+        } else if (opt == "--instrument-names") {
+            bInstrumentNamesOnly = true;
+        } else {
+            cerr << "Unknown option '" << opt << "'" << endl;
+            cerr << endl;
+            PrintUsage();
+            return EXIT_FAILURE;
         }
     }
-    FILE* hFile = fopen(argv[1], "r");
+    if (iArg >= argc) {
+        cout << "No file name provided!" << endl;
+        return EXIT_FAILURE;
+    }
+    const char* filename = argv[iArg];
+
+    FILE* hFile = fopen(filename, "r");
     if (!hFile) {
         cout << "Invalid file argument!" << endl;
         return EXIT_FAILURE;
     }
     fclose(hFile);
     try {
-        RIFF::File* riff = new RIFF::File(argv[1]);
+        RIFF::File* riff = new RIFF::File(filename);
         gig::File*  gig  = new gig::File(riff);
-        PrintFileInformations(gig);
-        cout << endl;
-        PrintGroups(gig);
-        cout << endl;
-        PrintSamples(gig);
-        cout << endl;
-        PrintScripts(gig);
-        cout << endl;
-        PrintInstruments(gig);
+
+        if (bRebuildChecksums) {
+            RebuildChecksumTable(gig);
+        } else if (bVerify) {
+            bool OK = VerifyFile(gig);
+            if (OK) cout << "All checks passed successfully! :-)\n";
+            return (OK) ? EXIT_SUCCESS : EXIT_FAILURE;
+        } else {
+            if (bInstrumentNamesOnly) {
+                PrintInstrumentNamesOnly(gig);
+            } else {
+                PrintFileInformation(gig);
+                cout << endl;
+                PrintGroups(gig);
+                cout << endl;
+                PrintSamples(gig);
+                cout << endl;
+                PrintScripts(gig);
+                cout << endl;
+                PrintInstruments(gig);
+            }
+        }
         delete gig;
         delete riff;
     }
@@ -92,8 +149,8 @@ int main(int argc, char *argv[])
     return EXIT_SUCCESS;
 }
 
-void PrintFileInformations(gig::File* gig) {
-    cout << "Global File Informations:" << endl;
+void PrintFileInformation(gig::File* gig) {
+    cout << "Global File Information:" << endl;
     cout << "    Total instruments: " << gig->Instruments << endl;
     if (gig->pVersion) {
        cout << "    Version: " << gig->pVersion->major   << "."
@@ -153,7 +210,7 @@ void PrintGroups(gig::File* gig) {
 void PrintSamples(gig::File* gig) {
     int samples = 0;
     cout << "ALL Available Samples (as there might be more than referenced by Instruments):" << endl;
-    gig::Sample* pSample = gig->GetFirstSample();
+    PubSample* pSample = (PubSample*) gig->GetFirstSample();
     while (pSample) {
         samples++;
         // determine sample's name
@@ -179,8 +236,28 @@ void PrintSamples(gig::File* gig) {
             cout << ", LoopFraction=" << pSample->LoopFraction << ", Start=" << pSample->LoopStart << ", End=" << pSample->LoopEnd;
             cout << ", LoopPlayCount=" << pSample->LoopPlayCount;
         }
-        cout << ", Length=" << pSample->SamplesTotal << " Compressed=" << ((pSample->Compressed) ? "true" : "false") << endl;
-        pSample = gig->GetNextSample();
+        cout << flush;
+        printf(", crc=%x", pSample->GetWaveDataCRC32Checksum());
+        fflush(stdout);
+        cout << ", Length=" << pSample->SamplesTotal << " Compressed=" << ((pSample->Compressed) ? "true" : "false")
+             << " foffset=" << pSample->pCkData->GetFilePos()
+             << " fsz=" << pSample->pCkData->GetSize()
+             << endl;
+#if 0
+        {
+            const uint bufSize = 64;
+            unsigned char buf[bufSize] = {};
+            pSample->SetPos(0);
+            RIFF::file_offset_t n = pSample->pCkData->Read(&buf[0], bufSize, 1);
+            //RIFF::file_offset_t n = pSample->Read(&buf[0], bufSize / pSample->FrameSize);
+            cout << "        FrameSize=" << pSample->FrameSize << ",Data[" << n << "]" << flush;
+            for (int x = 0; x < bufSize; ++x)
+                printf("%02x ", buf[x]);
+            printf("\n");
+            fflush(stdout);
+        }
+#endif
+        pSample = (PubSample*) gig->GetNextSample();
     }
 }
 
@@ -225,6 +302,18 @@ void PrintInstruments(gig::File* gig) {
     }
 }
 
+void PrintInstrumentNamesOnly(gig::File* gig) {
+    int instruments = 0;
+    gig::Instrument* pInstrument = gig->GetFirstInstrument();
+    for (; pInstrument; pInstrument = gig->GetNextInstrument()) {
+        instruments++;
+        string name = pInstrument->pInfo->Name;
+        if (name == "") name = "<NO NAME>";
+        else            name = '\"' + name + '\"';
+        cout << "Instrument " << instruments << ") " << name << endl;
+    }
+}
+
 void PrintRegions(gig::Instrument* instr) {
     int iRegion = 1;
     gig::Region* pRegion = instr->GetFirstRegion();
@@ -377,6 +466,25 @@ void PrintRegions(gig::Instrument* instr) {
     }
 }
 
+template<typename T_int>
+static void printIntArray(T_int* array, int size) {
+    printf("{");
+    for (int i = 0; i < size; ++i)
+        printf("[%d]=%d,", i, array[i]);
+    printf("}");
+    fflush(stdout);
+}
+
+static string lfoWaveName(gig::lfo_wave_t wave) {
+    switch (wave) {
+        case gig::lfo_wave_sine: return "Sine";
+        case gig::lfo_wave_triangle: return "Triangle";
+        case gig::lfo_wave_saw: return "Saw";
+        case gig::lfo_wave_square: return "Square";
+    }
+    return "Unknown";
+}
+
 void PrintDimensionRegions(gig::Region* rgn) {
     int dimensionRegions = 0;
     gig::DimensionRegion* pDimensionRegion;
@@ -398,8 +506,11 @@ void PrintDimensionRegions(gig::Region* rgn) {
         else {
             cout << "                Sample: <NO_VALID_SAMPLE_REFERENCE> " << endl;
         }
+        cout << "                LFO1WaveForm=" << lfoWaveName(pDimensionRegion->LFO1WaveForm) << ", LFO1Phase=" << pDimensionRegion->LFO1Phase << ", LFO1FlipPhase=" << pDimensionRegion->LFO1FlipPhase << endl;
         cout << "                LFO1Frequency=" << pDimensionRegion->LFO1Frequency << "Hz, LFO1InternalDepth=" << pDimensionRegion-> LFO1InternalDepth << ", LFO1ControlDepth=" << pDimensionRegion->LFO1ControlDepth << " LFO1Controller=" << pDimensionRegion->LFO1Controller << endl;
+        cout << "                LFO2WaveForm=" << lfoWaveName(pDimensionRegion->LFO2WaveForm) << ", LFO2Phase=" << pDimensionRegion->LFO2Phase << ", LFO2FlipPhase=" << pDimensionRegion->LFO2FlipPhase << endl;
         cout << "                LFO2Frequency=" << pDimensionRegion->LFO2Frequency << "Hz, LFO2InternalDepth=" << pDimensionRegion-> LFO2InternalDepth << ", LFO2ControlDepth=" << pDimensionRegion->LFO2ControlDepth << " LFO2Controller=" << pDimensionRegion->LFO2Controller << endl;
+        cout << "                LFO3WaveForm=" << lfoWaveName(pDimensionRegion->LFO3WaveForm) << ", LFO3Phase=" << pDimensionRegion->LFO3Phase << ", LFO3FlipPhase=" << pDimensionRegion->LFO3FlipPhase << endl;
         cout << "                LFO3Frequency=" << pDimensionRegion->LFO3Frequency << "Hz, LFO3InternalDepth=" << pDimensionRegion-> LFO3InternalDepth << ", LFO3ControlDepth=" << pDimensionRegion->LFO3ControlDepth << " LFO3Controller=" << pDimensionRegion->LFO3Controller << endl;
         cout << "                EG1PreAttack=" << pDimensionRegion->EG1PreAttack << "permille, EG1Attack=" << pDimensionRegion->EG1Attack << "s, EG1Decay1=" << pDimensionRegion->EG1Decay1 << "s,  EG1Sustain=" << pDimensionRegion->EG1Sustain << "permille, EG1Release=" << pDimensionRegion->EG1Release << "s, EG1Decay2=" << pDimensionRegion->EG1Decay2 << "s, EG1Hold=" << pDimensionRegion->EG1Hold << endl;
         cout << "                EG2PreAttack=" << pDimensionRegion->EG2PreAttack << "permille, EG2Attack=" << pDimensionRegion->EG2Attack << "s, EG2Decay1=" << pDimensionRegion->EG2Decay1 << "s,  EG2Sustain=" << pDimensionRegion->EG2Sustain << "permille, EG2Release=" << pDimensionRegion->EG2Release << "s, EG2Decay2=" << pDimensionRegion->EG2Decay2 << "s" << endl;
@@ -420,14 +531,108 @@ void PrintDimensionRegions(gig::Region* rgn) {
                 cout << "UNKNOWN - please report this !";
         }
         cout << ", VelocityResponseDepth=" << (int) pDimensionRegion->VelocityResponseDepth << ", VelocityResponseCurveScaling=" << (int) pDimensionRegion->VelocityResponseCurveScaling << endl;
-        cout << "                Pan=" << (int) pDimensionRegion->Pan << endl;
+        cout << "                VelocityUpperLimit=" << (int) pDimensionRegion->VelocityUpperLimit << " DimensionUpperLimits[]=" << flush;
+        printIntArray(pDimensionRegion->DimensionUpperLimits, 8);
+        cout << endl;
+#if 0 // requires access to protected member VelocityTable, so commented out for now
+        if (pDimensionRegion->VelocityTable) {
+            cout << "                VelocityTable[]=" << flush;
+            printIntArray(pDimensionRegion->VelocityTable, 127);
+            cout << endl;
+        }
+#endif
+        cout << "                Pan=" << (int) pDimensionRegion->Pan;
+        cout << ", SustainReleaseTrigger=";
+        switch (pDimensionRegion->SustainReleaseTrigger) {
+            case gig::sust_rel_trg_none:
+                cout << "NONE";
+                break;
+            case gig::sust_rel_trg_maxvelocity:
+                cout << "MAXVELOCITY";
+                break;
+            case gig::sust_rel_trg_keyvelocity:
+                cout << "KEYVELOCITY";
+                break;
+        }
+        cout << ", NoNoteOffReleaseTrigger=" << int(pDimensionRegion->NoNoteOffReleaseTrigger);
+        cout << endl;
+        {
+            gig::eg_opt_t& egopt = pDimensionRegion->EG1Options;
+            cout << "                EG1AttackCancel=" << egopt.AttackCancel << ", EG1AttackHoldCancel=" << egopt.AttackHoldCancel << ", EG1Decay1Cancel=" << egopt.Decay1Cancel << ", EG1Decay2Cancel=" << egopt.Decay2Cancel << ", EG1ReleaseCancel=" << egopt.ReleaseCancel << endl;
+        }
+        {
+            gig::eg_opt_t& egopt = pDimensionRegion->EG2Options;
+            cout << "                EG2AttackCancel=" << egopt.AttackCancel << ", EG2AttackHoldCancel=" << egopt.AttackHoldCancel << ", EG2Decay1Cancel=" << egopt.Decay1Cancel << ", EG2Decay2Cancel=" << egopt.Decay2Cancel << ", EG2ReleaseCancel=" << egopt.ReleaseCancel << endl;
+        }
 
         dimensionRegions++;
     }
 }
 
+struct _FailedSample {
+    gig::Sample* sample;
+    uint32_t calculatedCRC;
+};
+
+bool VerifyFile(gig::File* _gig) {
+    PubFile* gig = (PubFile*) _gig;
+
+    cout << "Verifying sample checksum table ... " << flush;
+    if (!gig->VerifySampleChecksumTable()) {
+        cout << "DAMAGED\n";
+        cout << "You may use --rebuild-checksums to repair the sample checksum table.\n";
+        return false;
+    }
+    cout << "OK\n" << flush;
+
+    cout << "Verifying samples ... " << flush;
+    std::map<int,_FailedSample> failedSamples;
+    int iTotal = 0;
+    for (gig::Sample* pSample = gig->GetFirstSample(); pSample; pSample = gig->GetNextSample(), ++iTotal) {
+        uint32_t crc; // will be set to the actually now calculated checksum
+        if (!pSample->VerifyWaveData(&crc)) {
+            _FailedSample failed;
+            failed.sample = pSample;
+            failed.calculatedCRC = crc;
+            failedSamples[iTotal] = failed;
+        }
+    }
+    if (failedSamples.empty()) {
+        cout << "ALL OK\n";
+        return true;
+    } else {
+        cout << failedSamples.size() << " of " << iTotal << " Samples DAMAGED:\n";
+        for (std::map<int,_FailedSample>::iterator it = failedSamples.begin(); it != failedSamples.end(); ++it) {
+            const int i = it->first;
+            gig::Sample* pSample = it->second.sample;
+
+            string name = pSample->pInfo->Name;
+            if (name == "") name = "<NO NAME>";
+            else            name = '\"' + name + '\"';
+
+            cout << "Damaged Sample " << (i+1) << ") " << name << flush;
+            printf(" expectedCRC=%x calculatedCRC=%x\n", pSample->GetWaveDataCRC32Checksum(), it->second.calculatedCRC);
+        }
+        return false;
+    }
+}
+
+void RebuildChecksumTable(gig::File* _gig) {
+    PubFile* gig = (PubFile*) _gig;
+
+    cout << "Recalculating checksums of all samples ... " << flush;
+    bool bSaveRequired = gig->RebuildSampleChecksumTable();
+    cout << "OK\n";
+    if (bSaveRequired) {
+        cout << "WARNING: File structure change required, rebuilding entire file now ..." << endl;
+        gig->Save();
+        cout << "DONE\n";
+        cout << "NOTE: Since the entire file was rebuilt, you may need to manually check all samples in this particular case now!\n";
+    }
+}
+
 string Revision() {
-    string s = "$Revision: 2609 $";
+    string s = "$Revision: 3623 $";
     return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
 }
 
@@ -439,8 +644,14 @@ void PrintVersion() {
 void PrintUsage() {
     cout << "gigdump - parses Gigasampler files and prints out the content." << endl;
     cout << endl;
-    cout << "Usage: gigdump [-v] FILE" << endl;
+    cout << "Usage: gigdump [-v | --verify | --rebuild-checksums] FILE" << endl;
     cout << endl;
-    cout << "	-v  Print version and exit." << endl;
+    cout << "   --rebuild-checksums  Rebuild checksum table for all samples." << endl;
+    cout << endl;
+    cout << "	-v                   Print version and exit." << endl;
+    cout << endl;
+    cout << "   --verify             Checks raw wave data integrity of all samples." << endl;
+    cout << endl;
+    cout << "   --instrument-names   Print only instrument names." << endl;
     cout << endl;
 }
